;; lirc-client.scm
;; lirc client library
;; $Id: lirc-client.scm,v 1.2 2006/01/31 22:27:08 jsb Exp $
;;
;; Copyright (c) 2006, Hans Bulfone
;; All rights reserved.
;;
;; Redistribution and use in source and binary forms, with or without
;; modification, are permitted provided that the following conditions are met:
;;
;;     * Redistributions of source code must retain the above copyright notice,
;;       this list of conditions and the following disclaimer.
;;     * Redistributions in binary form must reproduce the above copyright
;;       notice, this list of conditions and the following disclaimer in the
;;       documentation and/or other materials provided with the distribution.
;;     * Neither the name of the author nor the names of his contributors may
;;       be used to endorse or promote products derived from this software
;;       without specific prior written permission.
;;
;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
;; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
;; THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
;; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
;; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
;; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
;; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
;; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
;; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
;; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(declare
 (usual-integrations)
 (export lirc:init lirc:read-config lirc:next-code lirc:code->strings lirc:deinit)
 (foreign-declare
"#include <lirc/lirc_client.h>
#include <fcntl.h>
static struct lirc_config *the_lirc_config = NULL;
static int the_lirc_fd = -1;"))
(require-extension srfi-18)

(define (lirc:error message . args)
  (signal
   (make-composite-condition
    (make-property-condition 'exn 'message message 'arguments args)
    (make-property-condition 'lirc))))

(define lirc:init
  (let ((init
	 (foreign-lambda*
	  bool ((nonnull-c-string prog) (bool verbose))
	  "int r;"
	  "if(the_lirc_fd != -1)"
	  "  return(0);"
	  "the_lirc_fd = lirc_init(prog, verbose);"
	  "if(the_lirc_fd == -1)"
	  "  return(0);"
	  "r = fcntl(the_lirc_fd, F_GETFL, 0);"
	  "if(r == -1)"
	  "  {"
	  "    lirc_deinit();"
	  "    the_lirc_fd = -1;"
	  "    return(0);"
	  "  }"
	  "if(fcntl(the_lirc_fd, F_SETFL, r | O_NONBLOCK) == -1)"
	  "  {"
	  "    lirc_deinit();"
	  "    the_lirc_fd = -1;"
	  "    return(0);"
	  "  }"
	  "return(1);")))
    (lambda (prog verbose)
      (if (init prog verbose)
	  (void)
	  (lirc:error "lirc_init failed")))))

(define lirc:read-config
  (let ((read-config
	 (foreign-lambda*
	  bool ((c-string file))
	  "if(the_lirc_config != NULL)"
	  "  return(0);"
	  "return(lirc_readconfig(file, &the_lirc_config, NULL)==0);")))
    (lambda (file)
      (if (read-config file)
	  (void)
	  (lirc:error "lirc_readconfig failed")))))

(define lirc:next-code
  (let ((next-code (foreign-lambda int "lirc_nextcode" nonnull-c-pointer))
	(get-and-free-string
	 (foreign-lambda* c-string* ((nonnull-c-pointer ptr))
			  "return(ptr);")))
    (lambda ()
      (let-location ((code-ptr c-pointer #f))
	(let loop ()
	  (if (= 0 (next-code (location code-ptr)))
	      (if code-ptr
		  (get-and-free-string code-ptr)
		  (begin
		    (##sys#thread-block-for-i/o!
		     ##sys#current-thread
		     (foreign-value "the_lirc_fd" integer)
		     #t)
		    (thread-yield!)
		    (loop)))
	      (lirc:error "lirc_nextcode failed")))))))

(define lirc:code->strings
  (let ((code->string (foreign-lambda*
		       bool ((nonnull-c-string code)
			     (nonnull-c-pointer result))
		       "return(lirc_code2char(the_lirc_config, code, result)==0);"))
	(get-string (foreign-lambda* c-string ((nonnull-c-pointer ptr))
				     "return(ptr);")))
    (lambda (code)
      (let-location ((result-ptr c-pointer #f))
	(let loop ((result '()))
	  (if (code->string code (location result-ptr))
	      (if result-ptr
		  (loop (cons (get-string result-ptr)
			      result))
		  (reverse result))
	      (lirc:error "lirc_code2char failed")))))))

(define lirc:deinit
  (foreign-lambda*
   void ()
   "if(the_lirc_config)"
   "  {"
   "    lirc_freeconfig(the_lirc_config);"
   "    the_lirc_config = NULL;"
   "  }"
   "if(the_lirc_fd != -1)"
   "  {"
   "    lirc_deinit();"
   "    the_lirc_fd = -1;"
   "  }"))
